# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vulnerabilities.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='vulnerabilities.proto',
    package='',
    serialized_pb=_b(
        '\n\x15vulnerabilities.proto\"Z\n\rvulnerability\x12\r\n\x05title\x18\x02 \x02(\t\x12\x18\n\x10technical_detail\x18\x03 \x02(\t\x12\x13\n\x0brisk_rating\x18\x04 \x02(\t\x12\x0b\n\x03\x64na\x18\x05 \x02(\t\"v\n\x0fvulnerabilities\x12\x0f\n\x07scan_id\x18\x01 \x02(\x05\x12\x12\n\nagent_name\x18\x02 \x02(\t\x12\x15\n\ragent_version\x18\x03 \x02(\t\x12\'\n\x0fvulnerabilities\x18\x04 \x03(\x0b\x32\x0e.vulnerability')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_VULNERABILITY = _descriptor.Descriptor(
    name='vulnerability',
    full_name='vulnerability',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='title', full_name='vulnerability.title', index=0,
            number=2, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='technical_detail', full_name='vulnerability.technical_detail', index=1,
            number=3, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='risk_rating', full_name='vulnerability.risk_rating', index=2,
            number=4, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='dna', full_name='vulnerability.dna', index=3,
            number=5, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    options=None,
    is_extendable=False,
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=25,
    serialized_end=115,
)

_VULNERABILITIES = _descriptor.Descriptor(
    name='vulnerabilities',
    full_name='vulnerabilities',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='scan_id', full_name='vulnerabilities.scan_id', index=0,
            number=1, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='agent_name', full_name='vulnerabilities.agent_name', index=1,
            number=2, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='agent_version', full_name='vulnerabilities.agent_version', index=2,
            number=3, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='vulnerabilities', full_name='vulnerabilities.vulnerabilities', index=3,
            number=4, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    options=None,
    is_extendable=False,
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=117,
    serialized_end=235,
)

_VULNERABILITIES.fields_by_name['vulnerabilities'].message_type = _VULNERABILITY
DESCRIPTOR.message_types_by_name['vulnerability'] = _VULNERABILITY
DESCRIPTOR.message_types_by_name['vulnerabilities'] = _VULNERABILITIES

vulnerability = _reflection.GeneratedProtocolMessageType('vulnerability', (_message.Message,), dict(
    DESCRIPTOR=_VULNERABILITY,
    __module__='vulnerabilities_pb2'
    # @@protoc_insertion_point(class_scope:vulnerability)
))
_sym_db.RegisterMessage(vulnerability)

vulnerabilities = _reflection.GeneratedProtocolMessageType('vulnerabilities', (_message.Message,), dict(
    DESCRIPTOR=_VULNERABILITIES,
    __module__='vulnerabilities_pb2'
    # @@protoc_insertion_point(class_scope:vulnerabilities)
))
_sym_db.RegisterMessage(vulnerabilities)

# @@protoc_insertion_point(module_scope)
